//
//  ViewController.swift
//  VoltageRatioInput
//
//  Created by Phidgets on 2018-01-25.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController {

    let ch = VoltageRatioInput()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    var pickerArray = [String]()
    var bridgeGainArray = [String]()
    var sensorArray = [String]()
    
    @IBOutlet var voltageRatioLabel: UILabel!
    @IBOutlet var sensorValueTitle: UILabel!
    @IBOutlet var sensorValueLabel: UILabel!
    @IBOutlet var dataIntervalSlider: UISlider!
    @IBOutlet var dataIntervalLabel: UILabel!
    @IBOutlet var changeTriggerSlider: UISlider!
    @IBOutlet var changeTriggerLabel: UILabel!
    @IBOutlet var bridgeGainStackView: UIStackView!
    @IBOutlet var bridgeGainEnabled: UISwitch!
    @IBOutlet var bridgeGainTitle: UILabel!
    @IBOutlet var bridgeGainButton: UIButton!
    @IBOutlet var sensorTypeButton: UIButton!
    @IBOutlet var stackView: UIStackView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.voltageRatioChange.addHandler(voltageratio_change)
            let _ = ch.sensorChange.addHandler(sensor_change)
            try ch.setHubPort(0)
            try ch.setIsHubPortDevice(true)
            try ch.setDeviceSerialNumber(497042)
            try ch.open()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            let deviceID = try sender.getDeviceID()
            let channelSubClass = try sender.getChannelSubclass()
            let attachedDevice = sender as! VoltageRatioInput
            
            DispatchQueue.main.async {
                self.stackView.isHidden = false
                self.sensorValueLabel.isHidden = true
                self.sensorValueTitle.isHidden = true
                self.sensorTypeButton.isHidden = true
            }
            
            let minDataInterval = try Float(attachedDevice.getMinDataInterval())
            let maxDataInterval = try Float(attachedDevice.getMaxDataInterval())
            let dataInterval = try Float(attachedDevice.getDataInterval())
            let minVoltageRatioChangeTrigger = try Float(attachedDevice.getMinVoltageRatioChangeTrigger())
            let maxVoltageRatioChangeTrigger = try Float(attachedDevice.getMaxVoltageRatioChangeTrigger())
            let voltageRatioChangeTrigger = try Float(attachedDevice.getVoltageRatioChangeTrigger())
            
            if(channelSubClass == ChannelSubclass.voltageRatioInputBridge){
                let bridgeEnabled = try attachedDevice.getBridgeEnabled()
                DispatchQueue.main.async {
                    self.bridgeGainStackView.isHidden = false
                    self.bridgeGainButton.isHidden = false
                    self.sensorTypeButton.isHidden = true
                    self.sensorValueTitle.isHidden = true
                    self.sensorValueLabel.isHidden = true
                    self.bridgeGainEnabled.setOn((bridgeEnabled == true) ? true : false, animated: false)
                }
                bridgeGainArray.removeAll()
                if(deviceID == DeviceID.PN_1046){
                    bridgeGainArray = ["1x","8x","16x","32x","64x","128x"]
                }
                else if(deviceID == DeviceID.PN_DAQ1500){
                    bridgeGainArray = ["1x","2x","64x","128x"]
                }
            }
            else{
                DispatchQueue.main.async {
                    self.bridgeGainStackView.isHidden = true;
                    self.bridgeGainButton.isHidden = true
                }
            }
            
            DispatchQueue.main.async {
                if channelSubClass == ChannelSubclass.voltageRatioInputSensorPort{
                    self.sensorValueLabel.isHidden = false
                    self.sensorValueTitle.isHidden = false
                    self.sensorTypeButton.isHidden = false
                    self.sensorArray.removeAll()
                    self.sensorArray = ["Generic ratiometric sensor","1101 - IR Distance Adapter, with Sharp Distance Sensor 2D120X (4-30cm)","1101 - IR Distance Adapter, with Sharp Distance Sensor 2Y0A21 (10-80cm)","1101 - IR Distance Adapter, with Sharp Distance Sensor 2Y0A02 (20-150cm)","1102 - IR Reflective Sensor 5mm","1103 - IR Reflective Sensor 10cm","1104 - Vibration Sensor","1105 - Light Sensor","1106 - Force Sensor","1107 - Humidity Sensor","1108 - Magnetic Sensor","1109 - Rotation Sensor","1110 - Touch Sensor","1111 - Motion Sensor","1112 - Slider 60","1113 - Mini Joy Stick Sensor","1115 - Pressure Sensor","1116 - Multi-turn Rotation Sensor","1118 - 50Amp Current Sensor AC","1118 - 50Amp Current Sensor DC","1119 - 20Amp Current Sensor DC","1120 - FlexiForce Adapter","1121 - Voltage Divider","1122 - 30 Amp Current Sensor AC","1122 - 30 Amp Current Sensor DC","1124 - Precision Temperature Sensor","1125 - Humidity Sensor","1125 - Temperature Sensor","1126 - Differential Air Pressure Sensor +- 25kPa","1128 - MaxBotix EZ-1 Sonar Sensor","1129 - Touch Sensor","1131 - Thin Force Sensor","1134 - Switchable Voltage Divider","1136 - Differential Air Pressure Sensor +-2 kPa","1137 - Differential Air Pressure Sensor +-7 kPa","1138 - Differential Air Pressure Sensor 50 kPa","1139 - Differential Air Pressure Sensor 100 kPa","1140 - Absolute Air Pressure Sensor 20-400 kPa","1141 - Absolute Air Pressure Sensor 15-115 kPa","1146 - IR Reflective Sensor 1-4mm","3120 - Compression Load Cell (0-4.5 kg)","3121 - Compression Load Cell (0-11.3 kg)","3122 - Compression Load Cell (0-22.7 kg)","3123 - Compression Load Cell (0-45.3 kg)","3130 - Relative Humidity Sensor","3520 - Sharp Distance Sensor (4-30cm)","3521 - Sharp Distance Sensor (10-80cm)","3522 - Sharp Distance Sensor (20-150cm)"]
                }
                
                
                self.dataIntervalSlider.minimumValue = minDataInterval
                self.dataIntervalSlider.maximumValue = maxDataInterval
                self.dataIntervalSlider.value = dataInterval
                self.dataIntervalLabel.text = String(format:"%d ms", UInt32(dataInterval))
                
                
                if minVoltageRatioChangeTrigger == maxVoltageRatioChangeTrigger{
                    self.changeTriggerSlider.isEnabled = false
                }
                else{
                    self.changeTriggerSlider.minimumValue = minVoltageRatioChangeTrigger
                    self.changeTriggerSlider.maximumValue = maxVoltageRatioChangeTrigger
                    self.changeTriggerSlider.value = voltageRatioChangeTrigger
                    self.changeTriggerLabel.text = "\(voltageRatioChangeTrigger) V"
                }
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.stackView.isHidden = true
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }
    var timeStamp:TimeInterval = 0
    var lastTimeStamp:TimeInterval = 0
    func outputErrorMessage(errorTitle:String, errorMessage:String){
        timeStamp = NSDate.timeIntervalSinceReferenceDate
        //Add a delay for error events so the user does not get bombarded by error events
        if(timeStamp - lastTimeStamp > 5){
            let alert = UIAlertController(title: errorTitle, message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
            }))
            self.present(alert, animated: true, completion: nil)
            lastTimeStamp = timeStamp
        }
    }
    
    func sensor_change(sender: VoltageRatioInput, data: (sensorValue: Double, sensorUnit: UnitInfo)){
        DispatchQueue.main.async {
            self.sensorValueLabel.text = String(format:"%.3f%@", data.sensorValue, data.sensorUnit.symbol)
        }
    }
    
    func voltageratio_change(sender: VoltageRatioInput, voltageRatio: Double){
        do{
            if(try ch.getChannelSubclass() == ChannelSubclass.voltageRatioInputBridge){
                DispatchQueue.main.async {
                    self.voltageRatioLabel.text = String(format: "%.3EV/V", voltageRatio)
                }
            }
            else{
                DispatchQueue.main.async {
                    self.voltageRatioLabel.text = String(format: "%.3fV/V", voltageRatio)
                }
            }
            
            
        }catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: GUI controls
    @IBAction func setDataInterval(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setDataInterval(UInt32(slider.value))
            DispatchQueue.main.async {
                self.dataIntervalLabel.text = "\(UInt32(slider.value)) ms"
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setChangeTrigger(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setVoltageRatioChangeTrigger(Double(slider.value))
            DispatchQueue.main.async {
                self.changeTriggerLabel.text = String(format:"%0.3fV/V", Double(slider.value))
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: navigation
    @IBAction func pickerExit(_ segue: UIStoryboardSegue){
        dimBackground()
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
        else if segue.identifier == "bridgeGainSegue"{
            let vc:PickerViewController = segue.destination as! PickerViewController
            dimBackground()
            vc.titleName = "Bridge Gain"
            vc.pickerArray = bridgeGainArray
            vc.ch = ch
        }
        else if segue.identifier == "ratioSensorTypeSegue"{
            let vc:PickerViewController = segue.destination as! PickerViewController
            dimBackground()
            vc.titleName = "Sensor Type"
            vc.pickerArray = sensorArray
            vc.ch = ch
        }
    }
    
    //MARK: Misc
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    func dimBackground(){
        self.view.alpha = (self.view.alpha == 1.0) ? 0.5 : 1.0
    }


}

